package com.ho.test.perf.measure.commons.tools;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>Title: StopWatch</p>
 * 
 * <p>
 * Measures the duration of execution for a set of tasks.
 * <br/>Example of use:
 * <br/>
 * <br/>&nbsp;&nbsp;StopWatch.getInstance().start("task 1");
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 2");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 2bis");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;try {Thread.sleep(500);} catch (InterruptedException e) {}
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 2bis");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 2c");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;try {Thread.sleep(125);} catch (InterruptedException e) {}
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 2c");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 2d");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;try {Thread.sleep(30);} catch (InterruptedException e) {}
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 2d1").log("log message...");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;try {Thread.sleep(252);} catch (InterruptedException e) {}
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 2d1");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 2d");
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 2");
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 3");
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 3bis");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;try {Thread.sleep(40);} catch (InterruptedException e) {}
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 3bis");
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 3");
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().start("task 3bis");
 * <br/>&nbsp;&nbsp;&nbsp;&nbsp;try {Thread.sleep(250);} catch (InterruptedException e) {}
 * <br/>&nbsp;&nbsp;&nbsp;StopWatch.getInstance().stop("task 3bis");
 * <br/>&nbsp;&nbsp;StopWatch.getInstance().stop("task 1");
 * <br/>&nbsp;&nbsp;
 * <br/>&nbsp;&nbsp;// Displays the report of execution of StopWatch
 * <br/>&nbsp;&nbsp;System.out.println(StopWatch.getInstance());
 * 
 * </p>
 * 
 * @a2.Version
 */
public class StopWatch {
	// -------------------------------------------------------------- CONSTANTS
	// ----------------------------------------------------- PRIVATE ATTRIBUTES
	/**
	 * Contains the unique instance of the class.
	 */
	private static StopWatch instance = null;
	
	/**
	 * Contains all the tasks in the order of arrival.
	 */
	private Collection<StopWatchTask> tasks = new ArrayList<StopWatchTask>();
	
	/**
	 * Contains all tasks. The task name is used as key in the map.
	 */
	private Map<String, StopWatchTask> tasksByName = new HashMap<String, StopWatchTask>();
	
	/**
	 * Contains the current indentation level.
	 */
	private int ident = 0;
	
	// ------------------------------------------------------------- EXCEPTIONS
	// ------------------------------------------------------- INTERNAL CLASSES
	// ----------------------------------------------------------- CONSTRUCTORS
	/**
	 * Default constructor private: this is a singleton ...
	 */
	private StopWatch() {}

	// --------------------------------------------------------- PUBLIC METHODS
	/**
	 * Access method to the singleton. Returns the singleton.
	 */
	public final static synchronized StopWatch getInstance() {
		instance = null == instance ? new StopWatch() : instance;
		return instance;
	}
	
	/**
	 * Startup / create the task corresponding to the unique name of the given task.
	 * @param taskname
	 * @return
	 */
	public StopWatchTask start(String taskname) {
		/*
		// Try to retreive the corresponding task 
		StopWatchTask correspondingTask = null; 
		{
			correspondingTask = tasksByName.get(taskname);
			
			if (null == correspondingTask) {
				correspondingTask = null == correspondingTask ? new StopWatchTask(taskname, ident): correspondingTask;
				
				// The task doesn't exist, adds its reference in the lists of tasks
				tasks.add(correspondingTask);
				tasksByName.put(taskname, correspondingTask);
			} // end-if

		} // end-block
		*/
		
		// Try to retreive the corresponding task 
		StopWatchTask correspondingTask = new StopWatchTask(taskname, ident); 
		// The task doesn't exist, adds its reference in the lists of tasks
		tasks.add(correspondingTask);
		tasksByName.put(taskname, correspondingTask);

		
		// Start the task
		{
			correspondingTask.start();
			ident++;
		} // end-block
		
		return correspondingTask;
	}
	
	/**
	 * Stop the corresponding task to the unique reference of given task 
	 * @param taskname
	 * @return
	 */
	public StopWatchTask stop(String taskname) {
		StopWatchTask correspondingTask = tasksByName.get(taskname);
		correspondingTask.stop();
		ident--;
		//Output to the console because the measurements are performed 
		//   on asynchronously calls in controllers from the JSP layer PRESENTATION
		//System.out.println(correspondingTask.toString());
		
		return correspondingTask;
	}
	
	/**
	 * Specifies that a task is significant.
	 * @return The instance of the corresponding task.
	 */
	public StopWatchTask setSignificant(String taskName) {
		StopWatchTask correspondingTask = tasksByName.get(taskName);
		correspondingTask.setSignificant();
		return correspondingTask;
	}
	
	/**
	 * Clean the set of tasks
	 */
	public void clear() {
		tasks.clear(); 
		tasksByName.clear();
		ident=0;
	}
	
	/**
	 * Method toString
	 * @return
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		// Displaying the stack of execution of tasks
		String output = "StopWatchReport execution Stack \n";
		for (StopWatchTask currTask : tasks) {
			output += currTask + "\n";
		}
		
		// Showing the significant tasks
		output += "\nStopWatchReport significant task \n";
		for (StopWatchTask currTask : tasks) {
			if (currTask.isSignificant()) {
				output += currTask.dumpWithoutIdent() + "\n";	
			}
		}
		
		return output;
	}
	// -------------------------------------------------------- PRIVATE METHODS
	// ------------------------------------------------------ PROTECTED METHODS
	// --------------------------------------------- ADVANCED GETTERS & SETTERS
	// ------------------------------------------------------ GETTERS & SETTERS
}
